@@ -2,6 +2,7 @@ from torch.utils.tensorboard import SummaryWriter
 import torch.optim as optim
 from torch.utils.data import random_split
 import torch
+import matplotlib.pyplot as plt
 import time
 from ingradient_library.dataloads import *
 from ingradient_library.model import *
@@ -37,7 +38,11 @@ class Trainer(object):
             deep_supervision_size = output.shape[1]
             channel_size = output.shape[2]
             output = output.reshape(batch_size * deep_supervision_size, channel_size, -1)
-            target = target.reshape(batch_size, 1, -1).repeat(1, deep_supervision_size,1).reshape(batch_size * deep_supervision_size, -1)
+            if channel_size == 1:
+                target = target.reshape(batch_size, 1, -1).repeat(1, deep_supervision_size,1).reshape(batch_size * deep_supervision_size, 1, -1)
+                target = target.float()
+            else:
+                target = target.reshape(batch_size, 1, -1).repeat(1, deep_supervision_size,1).reshape(batch_size * deep_supervision_size, -1)
             weight = torch.ones(deep_supervision_size).to(output.device.index)
             weight.requires_grad = False
             acc_value = self.deep_supervision_weight
@@ -51,21 +56,28 @@ class Trainer(object):
             batch_size = output.shape[0]
             channel_size = output.shape[1]
             output = output.reshape(batch_size, channel_size, -1)
-            target = target.reshape(batch_size, -1)
+            if channel_size == 1:
+                target = target.float()
+                target = target.reshape(batch_size, 1, -1)
+            
+            else:
+                target = target.reshape(batch_size, -1)
 
         for i in range(len(self.losses)):
-            loss = self.losses[i]
-            temp = loss(output, target) * torch.tensor(self.regulizers[i]).to(output.device.index)
+            loss = self.losses[i] #여러 Loss들에 대해
+            print("output, target", output.shape, target.shape)
+            temp = loss(output, target) * torch.tensor(self.regulizers[i]).float().to(output.device.index)
+            print('before', temp.shape)
             for _ in range(len(temp.shape)-1):
                 temp = temp.mean(-1)
-            writer.add_scalar('Iteration\Loss_'+ str(type(loss)), temp[0], epoch)
+            writer.add_scalar('Iteration\Loss_'+ str(type(loss)), temp[0], epoch) #(batch, -1) 로 나오게끔 resize
+            print('after', temp.shape)
             if self.is_deep_supervision == True:
                 temp = temp.view(batch_size, deep_supervision_size, -1)
             if i == 0:
                 losses = temp
             else:
                 losses += temp
-
         
         if self.is_deep_supervision == True:
             result = (losses * weight).mean()
@@ -111,11 +123,21 @@ class Trainer(object):
                 else:
                     output = self.model(images)
                     self.optimizer.zero_grad()
+                    print(self.tr_dl.current_index)
                     loss = self.calculate_loss(output, seg, writer, e)
-                    if self.tr_dl.current_index < 20:
-                        deep_supervision_visualization(output, seg)
                     loss.backward()
                     self.optimizer.step()
+                
+                if not self.is_deep_supervision:
+                    plt.imshow(torch.argmax(output[0,:,output.shape[2]//2], dim = 0).detach().cpu(), vmax =1 , vmin= 0)
+                    plt.show()
+                else:
+                    plt.imshow(torch.argmax(output[0,0,:,output.shape[2]//2], dim = 0).detach().cpu(), vmax =1 , vmin= 0)
+                    plt.show()
+                plt.imshow(images[0,0,output.shape[2]//2].detach().cpu())
+                plt.show()
+                plt.imshow(seg[0,output.shape[2]//2].detach().cpu(), vmax =1 , vmin= 0)
+                plt.show()
 
 
             if self.val_dl != None:
@@ -171,9 +193,7 @@ class Interactive_Trainer(Trainer):
                 train_loss = loss.item()
                 
                 center = output.shape[3] // 2
-                if self.tr_dl.current_index % visualization_ineterval == 0:
-                    print("points : ", 0)
-                    visualization(output, seg)
+                
                 loss.backward()
                 self.optimizer.step()
 
